#include "stdafx.h"
#include "Database.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CDatabase g_DB;

bool CDatabase::MountVol()
{
	// Mount a volume
	BOOL bResult = CeMountDBVol(
		&m_guid,			// guid of mount
		TEXT("\\CEDB.cdb"),	// full path and name of database volume
		OPEN_ALWAYS);		// flags

	// return true/false based on resultant value
	return bResult ? true : false;
}

bool CDatabase::OpenDB()
{
	// Open the database
	m_hDB = CeOpenDatabaseEx(
		&m_guid,			// guid of mounted database
		&m_oid,				// object identifier of database
		TEXT("CEDB"),		// name of database
		0,					// Property id for sort order
		0/*CEDB_AUTOINCREMENT*/,	// auto increment record pointer on ReadRecord
		NULL);				// do not receive notifications

	// return true/false based on resultant value
	if(m_hDB != INVALID_HANDLE_VALUE)
		return true;
	else 
		return false;
}

bool CDatabase::CreateDB()
{
	// Create the database
	CEDBASEINFO ced;
	ZeroMemory(&ced, sizeof(CEDBASEINFO));
	ced.dwSize = sizeof(CEDBASEINFO);
	ced.dwFlags = CEDB_VALIDNAME;
	wcscpy(ced.szDbaseName, TEXT("CEDB"));
	
	m_oid = CeCreateDatabaseEx(
		&m_guid,			// guid of mounted database
		&ced);				// database information

	// return true/false based on resultant value
	if(m_oid)
		return true;
	else
		return false;
}

bool CDatabase::SeekDB(DWORD dwSeekType, DWORD dwValue)
{
	// Seek to beginning of database
	DWORD dwIndex;
	CEOID oid = CeSeekDatabase(m_hDB, dwSeekType, dwValue, &dwIndex);

	// return true/false based on resultant value
	if(oid)
		return true;
	else
		return false;
}


CDatabase::CDatabase()
{
	bool bResult;

	// Mount database volume
	bResult = MountVol();
	ASSERT(bResult);

	// Open the database
	bResult = OpenDB();
	if(!bResult)
	{	// If not create the database
		bResult = CreateDB();
		ASSERT(bResult);
		
		// Open the database
		bResult = OpenDB();
		ASSERT(bResult);
	}

	// Seek to the beginning of a populated database
	bResult = SeekDB(CEDB_SEEK_BEGINNING);
	//ASSERT(bResult);
}

CDatabase::~CDatabase()
{
	// Unmount the volume
	BOOL bResult = CeUnmountDBVol(&m_guid);
	ASSERT(bResult);
}


bool CDatabase::ReadRecord(CRecord &record)
{
	BYTE *prgBuffer = NULL;
	DWORD dwBuffer = 0;
	WORD wPropId = 0;

	CEOID oid = CeReadRecordPropsEx(
		m_hDB,				// handle to database
		CEDB_ALLOWREALLOC,	// flags
		&wPropId,			// read all properties
		NULL,				// read all properties
		&prgBuffer,			// pointer to buffer
		&dwBuffer,			// sizeof buffer
		NULL);				// parameter not meaningful

	if(oid)
	{
		record.PropValToRecord((CEPROPVAL *) prgBuffer);
		delete prgBuffer;

		return true;
	}
	else
	{
		delete prgBuffer;

		return false;
	}
}

bool CDatabase::AddRecord(CRecord record)
{
	CEPROPVAL *prgPropVal = record.NewPropVal();
	CEOID oid = CeWriteRecordProps(
		m_hDB,				// handle to database
		0,					// Create new record
		5,					// five properties
		prgPropVal);		// propval structure
	record.DeletePropVal(prgPropVal);

	// return true/false based on resultant value
	if(m_oid)
		return true;
	else
		return false;
}

bool CDatabase::DeleteRecord(int nIndex)
{
	// Seek to beginning of database
	DWORD dwIndex;
	CEOID oid = CeSeekDatabase(m_hDB, CEDB_SEEK_BEGINNING, nIndex, &dwIndex);
	ASSERT(oid);

	// Delete the record
	BOOL bResult = CeDeleteRecord(m_hDB, oid);
	
	// return true/false based on resultant value
	return bResult ? true : false;
}
